/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.engine.xslt.sdk;

import java.io.File;
import java.net.URI;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessagingException;
import javax.xml.transform.Source;
import javax.xml.transform.TransformerException;

import org.objectweb.petals.component.common.HandlingException;
import org.objectweb.petals.component.common.MEPConstants;
import org.objectweb.petals.component.common.se.AbstractServiceEngine;
import org.objectweb.petals.component.common.su.SimpleServiceUnitManager;
import org.objectweb.petals.component.common.util.MessageExchangeWrapper;
import org.objectweb.petals.component.common.util.PetalsExtensionsUtil;
import org.objectweb.petals.tools.jbicommon.descriptor.Extensions;
import org.objectweb.petals.tools.jbicommon.util.StringHelper;

/**
 * 
 * @author ofabre
 *
 */
public class XsltEngine extends AbstractServiceEngine {

    private Xslt xslt;

    private final static String XSL = "xsl";

    @Override
    protected boolean onExchange(MessageExchangeWrapper exchange,
        Extensions extensions) throws HandlingException {

        // Get Operation name
        String opName = exchange.getOperationName();

        // Get Message Exchange Pattern
        URI pattern = exchange.getExchangePattern();

        // Get in Message content
        Source inContent = exchange.getInMessageContentAsSource(); 

        if ("transform".equalsIgnoreCase(opName)
            && MEPConstants.IN_OUT_PATTERN.value().equals(pattern)) {

            String xslPath = retrieveXSLPath(exchange, extensions);

            try {
                Source outContent = xslt.transform(inContent, xslPath);
                exchange.setOutMessageContent(outContent);
            } catch (TransformerException e) {
                throw new HandlingException("Error transforming message", e);
            } catch (MessagingException e) {
                throw new HandlingException("Error transforming message", e);
            }

        } else {
            throw new HandlingException("Operation not allowed ("
                + exchange.getOperation().getLocalPart()
                + ") with the following pattern ("
                + exchange.getPattern().toString() + ")");
        }

        return true;
    }

    private String retrieveXSLPath(MessageExchangeWrapper exchange, Extensions extensions) throws HandlingException {
        String suRootPath = ((SimpleServiceUnitManager) getServiceUnitManager())
            .getSUInstallRootForActivatedEp(exchange.getEndpoint());
        if (!suRootPath.endsWith(File.separator)) {
            suRootPath = suRootPath + File.separator;
        }

        String xslRelativePath = PetalsExtensionsUtil
            .extractValueFromKeyValueExtension(extensions, XSL);
        if(xslRelativePath.startsWith(File.separator)){
            xslRelativePath = xslRelativePath.substring(1);
        }

        if (StringHelper.isNullOrEmpty(xslRelativePath)) {
            throw new HandlingException(
                "You must specify the relative path of "
                    + "your xsl file into your provides nodes");
        }

        String xslPath = suRootPath + xslRelativePath;
        return xslPath;
    }

    @Override
    public void start() throws JBIException {
        super.start();
        xslt = new XsltImpl();
    }

    @Override
    public void stop() throws JBIException {
        super.stop();
        xslt = null;
    }

}
